# coding: utf-8
from django.shortcuts import render
from django.db.models import Count
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.request import Request
from rest_framework.response import Response
from inspect_report.models import Tasks, CheckSession
import json
from datetime import datetime, timedelta
from config.config import TABLE_PRE
from django.views.decorators.clickjacking import xframe_options_exempt
from inspect_report import permissions
import logging
from functools import reduce


logger = logging.getLogger(__name__)


class TasksApi(viewsets.ViewSet):
    authentication_classes = ()
    permission_classes = ()
    # permission_classes = (permissions.ValidateToken, )

    @action(['get'], detail=False)
    def obtain(self, req: Request):
        """
        查询任务列表
        :param req:
        :return:
        """
        tasks = Tasks.objects.all().order_by('-createdAt').values('id', 'name')
        # return render(req, 'inspect/index.html', {'tasks': tasks})
        return Response({'code': 0, 'msg': 'success', 'data': tasks})

    @action(['get'], detail=False)
    def seat(self, req: Request):
        """
        获取坐席列表
        :param req:
        :return:
        """
        task_id = req.GET.get('task', '')
        task_condition = {'hasCheck': 1}
        session_condition = {}
        if task_id:
            task_condition['name__startswith'] = task_id
        tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
        task_dict = {}
        for t in tasks:
            if t['sessionCollectionId'] in task_dict.keys():
                task_dict[t['sessionCollectionId']].append(t['id'])
            else:
                task_dict[t['sessionCollectionId']] = [t['id'], ]
        return_data = []

        for k, v in task_dict.items():
            # session_collection = t['sessionCollectionId']
            table_name = TABLE_PRE + k
            tn = CheckSession.set_table(table_name)
            session_condition['taskId__in'] = v
            agents = tn.objects.filter(**session_condition).values('agentName')
            # .annotate(Count=Count('agentName')).values('agentName')
            # if agents:
            #     return_data.append(agents)
            for agent in agents:
                if agent not in return_data:
                    return_data.append(agent)
        # data_reduce = lambda x, y: x if y in x else x + [y]
        # return_data = reduce(data_reduce, [[], ] + return_data)
        return Response({'code': 0, 'msg': 'success', 'data': return_data})

    @action(['post'], detail=False)
    def rule(self, req: Request):
        """
        获取违规项分析
        :param req:
        :return:
        """
        task_id = req.data.get('task', '')
        agent_name = req.data.get('agentName', '')
        start_date = req.data.get('start_date', (datetime.now()+timedelta(days=-1)).strftime('%Y-%m-%d'))
        end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
        task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
        if task_id:
            task_condition['name__startswith'] = task_id
        tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
        task_dict = {}
        for t in tasks:
            if t['sessionCollectionId'] in task_dict.keys():
                task_dict[t['sessionCollectionId']].append(t['id'])
            else:
                task_dict[t['sessionCollectionId']] = [t['id'], ]
        return_data = {}
        for k, v in task_dict.items():
            # session_collection = t.sessionCollectionId
            table_name = TABLE_PRE + k
            tn = CheckSession.set_table(table_name)
            session_condition = {'taskId__in': v}
            if agent_name:
                session_condition['agentName'] = agent_name
            checks = tn.objects.filter(**session_condition).values('id', 'checkResult')
            # checks = tn.objects.filter(**session_condition).values('id', 'checkResult')
            for check in checks:
                result = check['checkResult']
                # session_id = check['id']
                if result:
                    data = json.loads(result)
                    for d in data:
                        if 'isViolation' in d.keys() and d['isViolation'] and 'rule' in d.keys() and 'name' in d['rule'].keys():
                            name = d['rule']['name']
                            if name in return_data.keys():
                                return_data[name]['count'] += 1
                                # return_data[name]['ids'].append(session_id)
                            else:
                                return_data[name] = {'count': 1}
                                # return_data[name] = {'count': 1, 'ids': [session_id, ]}
        data_sort = []
        for k, v in return_data.items():
            v['rule'] = k
            data_sort.append(v)
        data_sort = sorted(data_sort, key=lambda x: x['count'], reverse=True)
        return Response({'code': 0, 'msg': 'success', 'data': data_sort})

    @action(['post'], detail=False)
    def seat_session(self, req: Request):
        """
        坐席违规分析-按照会话
        :param req:
        :return:
        """
        task_id = req.data.get('task', '')
        num = req.data.get('num', '10')
        start_date = req.data.get('start_date', (datetime.now()+timedelta(days=-1)).strftime('%Y-%m-%d'))
        end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
        task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
        if task_id:
            task_condition['name__startswith'] = task_id
        tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
        task_dict = {}
        for t in tasks:
            if t['sessionCollectionId'] in task_dict.keys():
                task_dict[t['sessionCollectionId']].append(t['id'])
            else:
                task_dict[t['sessionCollectionId']] = [t['id'], ]
        return_data = {}
        for k, v in task_dict.items():
            table_name = TABLE_PRE + k
            tn = CheckSession.set_table(table_name)
            session_condition = {'taskId__in': v}
            checks = tn.objects.filter(**session_condition).values('agentName')
            for check in checks:
                agent_name = check['agentName']
                if agent_name:
                    if agent_name in return_data.keys():
                        return_data[agent_name] += 1
                    else:
                        return_data[agent_name] = 1
        data_max = sorted(return_data.items(), key=lambda item: item[1], reverse=False)
        data_max = data_max[-int(num):]
        data_max_return = {}
        for dm in data_max:
            data_max_return[dm[0]] = dm[1]
        return Response({'code': 0, 'msg': 'success', 'data': data_max_return})

    @action(['post'], detail=False)
    def seat_score(self, req: Request):
        """
        坐席违规分析-按照得分
        :param req:
        :return:
        """
        task_id = req.data.get('task', '')
        num = req.data.get('num', '10')
        start_date = req.data.get('start_date', (datetime.now()+timedelta(days=-1)).strftime('%Y-%m-%d'))
        end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
        task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
        if task_id:
            task_condition['name__startswith'] = task_id
        tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
        task_dict = {}
        for t in tasks:
            if t['sessionCollectionId'] in task_dict.keys():
                task_dict[t['sessionCollectionId']].append(t['id'])
            else:
                task_dict[t['sessionCollectionId']] = [t['id'], ]
        return_data = {}
        for k, v in task_dict.items():
            table_name = TABLE_PRE + k
            tn = CheckSession.set_table(table_name)
            session_condition = {'taskId__in': v}
            checks = tn.objects.filter(**session_condition).values('agentName', 'score')
            for check in checks:
                agent_name = check['agentName']
                if agent_name:
                    if agent_name in return_data.keys():
                        return_data[agent_name] += check['score']
                    else:
                        return_data[agent_name] = check['score']
        data_max = sorted(return_data.items(), key=lambda item: item[1], reverse=False)
        data_max = data_max[:int(num)]
        data_max_return = {}
        for dm in data_max:
            data_max_return[dm[0]] = dm[1]
        return Response({'code': 0, 'msg': 'success', 'data': data_max_return})

    @action(['post'], detail=False)
    def seat_rule(self, req: Request):
        """
        坐席违规分析-按照得分
        :param req:
        :return:
        """
        task_id = req.data.get('task', '')
        num = req.data.get('num', '10')
        a_name = req.data.get('agentName', '')
        sort = req.data.get('sort', 'des')
        start_date = req.data.get('start_date', (datetime.now()+timedelta(days=-1)).strftime('%Y-%m-%d'))
        end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
        task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
        if task_id:
            task_condition['name__startswith'] = task_id
        tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
        task_dict = {}
        for t in tasks:
            if t['sessionCollectionId'] in task_dict.keys():
                task_dict[t['sessionCollectionId']].append(t['id'])
            else:
                task_dict[t['sessionCollectionId']] = [t['id'], ]
        return_data = {}
        for k, v in task_dict.items():
            table_name = TABLE_PRE + k
            tn = CheckSession.set_table(table_name)
            session_condition = {'taskId__in': v}
            if a_name:
                session_condition['agentName'] = a_name
            checks = tn.objects.filter(**session_condition).values('agentName', 'violationRuleCount')
            for check in checks:
                agent_name = check['agentName']
                if agent_name:
                    if agent_name in return_data.keys():
                        return_data[agent_name]['total_session'] += 1
                        if check['violationRuleCount'] > 0:
                            return_data[agent_name]['validate_session'] += 1
                    else:
                        if check['violationRuleCount'] > 0:
                            return_data[agent_name] = {'total_session': 1, 'validate_session': 1}
                        else:
                            return_data[agent_name] = {'total_session': 1, 'validate_session': 0}
        # data_max = sorted(return_data.items(), key=lambda item: item[1], reverse=False)
        pre_data_sort = []
        for k, v in return_data.items():
            ratio = round(v['validate_session']/v['total_session'], 2)
            v['ratio'] = ratio
            v['agentName'] = k
            pre_data_sort.append(v)
        if sort == 'asc':
            data_sort = sorted(pre_data_sort, key=lambda x: x['ratio'])
            for i in range(len(data_sort)):
                data_sort[i]['ranking'] = len(data_sort) - i
        else:
            data_sort = sorted(pre_data_sort, key=lambda x: x['ratio'], reverse=True)
            for i in range(len(data_sort)):
                data_sort[i]['ranking'] = i + 1
                data_sort[i]['ratio'] = '{0}%'.format(int(data_sort[i]['ratio'] * 100))
        return Response({'code': 0, 'msg': 'success', 'data': data_sort})

    @action(['post'], detail=False)
    def static(self, req: Request):
        """
        获取违规项分析
        :param req:
        :return:
        """
        task_id = req.data.get('task', '')
        start_date = req.data.get('start_date', (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d'))
        end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
        task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
        if task_id:
            task_condition['name__startswith'] = task_id
        tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
        task_dict = {}
        for t in tasks:
            if t['sessionCollectionId'] in task_dict.keys():
                task_dict[t['sessionCollectionId']].append(t['id'])
            else:
                task_dict[t['sessionCollectionId']] = [t['id'], ]
        return_data = {'total_session': 0, 'validate_session': 0}
        for k, v in task_dict.items():
            table_name = TABLE_PRE + k
            tn = CheckSession.set_table(table_name)
            session_condition = {'taskId__in': v}
            checks = tn.objects.filter(**session_condition).values('violationRuleCount')
            for check in checks:
                return_data['total_session'] += 1
                if check['violationRuleCount'] > 0:
                    return_data['validate_session'] += 1
        if return_data['total_session']:
            return_data['ratio'] = round(return_data['validate_session']/return_data['total_session'], 2)
        else:
            return_data['ratio'] = 0.0
        return Response({'code': 0, 'msg': 'success', 'data': return_data})

    @action(['post'], detail=False)
    def static_date(self, req: Request):
        """
        获取违规项分析
        :param req:
        :return:
        """
        task_id = req.data.get('task', '')
        start_date = req.data.get('start_date', (datetime.now() + timedelta(days=-6)).strftime('%Y-%m-%d'))
        end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
        task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
        if task_id:
            task_condition['name__startswith'] = task_id
        tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
        task_dict = {}
        for t in tasks:
            if t['sessionCollectionId'] in task_dict.keys():
                task_dict[t['sessionCollectionId']].append(t['id'])
            else:
                task_dict[t['sessionCollectionId']] = [t['id'], ]
        return_data = {}
        for k, v in task_dict.items():
            table_name = TABLE_PRE + k
            tn = CheckSession.set_table(table_name)
            session_condition = {'taskId__in': v}
            checks = tn.objects.filter(**session_condition).values('violationRuleCount', 'createdAt')
            for check in checks:
                date = check['createdAt'].strftime('%Y-%m-%d')
                if date in return_data.keys():
                    return_data[date]['total_session'] += 1
                    if check['violationRuleCount'] > 0:
                        return_data[date]['validate_session'] += 1
                else:
                    if check['violationRuleCount'] > 0:
                        return_data[date] = {'total_session': 1, 'validate_session': 1}
                    else:
                        return_data[date] = {'total_session': 1, 'validate_session': 0}
        data_sort = []
        for k, v in return_data.items():
            # return_data[k]['ratio'] = round(v['validate_session']/v['total_session'], 2)
            if v['total_session']:
                v['ratio'] = round(v['validate_session']/v['total_session'], 2)
            else:
                v['ratio'] = 0
            v['date'] = k
            data_sort.append(v)
        data_sort = sorted(data_sort, key=lambda x: x['date'], reverse=True)
        return Response({'code': 0, 'msg': 'success', 'data': data_sort})


@xframe_options_exempt
def rule(request):
    return render(request, 'inspect/rule.html')


@xframe_options_exempt
def seat(request):
    return render(request, 'inspect/seat.html')


@xframe_options_exempt
def group(request):
    return render(request, 'inspect/group.html')
